<?php

class ProcessManager
{
    // 親プロセスから起動した際に取得した子プロセスの PID を取得します。
    protected $pids = [];

    // 現在のプロセスが起動されている PID を取得します。
    // 子プロセスの場合常に 0 になります。
    protected $childProcessId = null;

    public function run(): void
    {
        // 子プロセスから更に子プロセスを起動して無限にプロセスを起動する状態に陥らないようにするために
        // 自身が生成された子プロセスである場合、 run の実行をサーキット・ブレイクします。
        if ($this->childProcessId === 0) {
            return;
        }

        // プロセスをフォークします。
        $this->childProcessId = pcntl_fork();
        if ($this->childProcessId === 0) {
            // 子プロセスの場合

            // 重たい処理を走らせる
            $this->heavyProcessInChildProcess();

            // この処理以降は親プロセスのみで行えばいいので子プロセスである場合、
            // 正常に終了した旨のシグナルを親プロセスに送ります。
            exit(0);
        } elseif ($this->childProcessId > 0) {
            // 親プロセスの場合
            echo "子プロセス {$this->childProcessId} が起動しました。\n";

            // 後ほど子プロセス達から終了シグナルが送られるのを待機するために使用するため PID を親プロセスの ProcessManager に格納します。
            $this->pids[] = $this->childProcessId;
        }
    }

    protected function heavyProcessInChildProcess(): void
    {
        // pcntl_fork だと子プロセス自身が自分の PID を得ることは出来ませんが、
        // getmypid を使用すると子プロセス自身が自分の PID を得ることが可能です。
        $currentPID = getmypid();

        echo "子プロセス {$currentPID} の処理を開始しました。\n";

        // 重たい処理
        sleep(10);

        echo "子プロセス {$currentPID} の処理が終了しました。\n";
    }

    public function getPIDs(): array
    {
        return $this->pids;
    }
}

// 実行時間の計測を開始します
$start = microtime(true);

// ProcessManager のインスタンスを起動します。
$processManager = new ProcessManager();

// 子プロセスの起動を指定した回数だけ行います。この場合 3 つの子プロセスを起動します。
for ($i = 0; $i < 3; $i++) {
    $processManager->run();
}

// それぞれのプロセスが終了するまで待機します。
foreach ($processManager->getPIDs() as $pid) {
    echo "プロセス {$pid} から終了シグナルが送られるのを待機します。\n";

    // 指定した PID の終了を待つ場合は pcntl_wait ではなく pcntl_waitpid を使用すると可能です。
    pcntl_waitpid($pid, $status);
    echo "プロセス {$pid} が終了しました\n";
}

// 実際経過した時間を出力します。
echo "実行時間: " . (microtime(true) - $start) . "\n";