<?php
// コルーチン内で非同期処理を有効にします。
\Swoole\Runtime::enableCoroutine();

function guaranteeRead($client, int $length) {
    // GET など、リクエストボディを送らないメソッドの場合 Content-Length が 0 になるので、早期リターンさせます。
    if ($length <= 0) {
        return '';
    }

    // 読み込み終わった文字列を格納しておくための変数を用意します。
    $read = '';

    // 残り読み込む必要のある文字列の長さを格納しておきます。
    // 本来期待する読み込む文字列の長さは $length であるため、初期値として入れておきます。
    $remaining = $length;
    do {
        // 読み込んだ値を一旦バッファという形でおいておきます。
        $buffer = fread($client, $length);

        // 読み込んだ文字列を読み込み済みのデータを変数に追記します。
        $read .= $buffer;

        // 読み込んだ文字列の長さを $remaining から減算します。
        // そうすると最終的に $remaining は 0 になるため do-while 文を抜けます。
        $remaining -= strlen($buffer);
    } while ($remaining > 0);

    return $read;
}

function parseClient($client) {
    $headers = [];
    while ($line = fgets($client)) {
        // \n または \r\n の行がでるまで繰り返します
        if ($line === "\n" || $line === "\r\n") {
            break;
        }

        $headers[] = $line;
    }

    // headers を上書きします。
    $headers = array_reduce(
        // 最初の値を含まない値を取得し、以後 key => [ value1, value2, ...valueN ] のような形にします。
        array_slice($headers, 1),
        function ($carry, $header) {

            // 読み込んだ行を配列に入れます。
            [$headerName, $headerValue] = explode(
            // : で区切るようにします。
                ':',

                // 取得した行を渡します。
                $header,

                // : の左辺と右辺のみを取得するため 2 を指定します。
                // これを指定していないと、リクエストヘッダーの値の中に : が含まれているとそこも分割されてしまうためです。
                2
            );

            // キーは左右の余分な空白を除去し、英字を小文字にします。 RFC ではリクエストヘッダー名の大小は区別しないという定義がなされているためです。[^4]
            // また値についても同様に左右の空白や改行を除去するために trim を実行します。値は Set-Cookie のように複数行渡される場合も考慮し、複数の値を保持できるようにしておきます。
            $carry[strtolower(trim($headerName))][] = trim($headerValue);
            return $carry;
        },
        // 先頭の配列は添字と値に分ける必要がないため、最初に入れておきます。
        [trim($headers[0] ?? '')]
    );

    [$method, $path, $protocol] = explode(' ', $headers[0] ?? '');

    $contentLength = max(
        // 0 以上の値にするため min を使用して 0 以上になるように保証をします。
        0,

        // Content-Length の値は文字列なためそれを int にキャストします。
        (int) ($headers['content-length'][0] ?? 0)
    );


    $requestBody = guaranteeRead($client, $contentLength);

    sleep(3);

    // ステータスコードを返します。
    fwrite($client, "HTTP/1.1 200 OK\n");

    // close を返返します。
    fwrite($client, "Connection: close\n");

    // Content-Type を返します。
    fwrite($client, "Content-Type: text/html; charset=UTF-8\n");

    $responseBody = '';
    if ($path === '/' || $path === '/index.html') {
        // POST で送られた場合と、そうではない場合で分岐します。
        if ($method === 'POST') {
            $responseBody = "{$requestBody}を受け取りました！";
        } else {
            $responseBody = "トップページにアクセスしました！";
        }
    } elseif ($path === '/about.html') {
        $responseBody = "Swoole で楽しく非同期処理！";
    } else {
        $responseBody = "おっと！ページが存在しないようです。";
    }

    fwrite($client, "Content-Length: " . strlen($responseBody) . "\n");
    fwrite($client, "\n");
    fwrite($client, $responseBody);

    printf("%s にレスポンスを書き込みました。\n", stream_socket_get_name($client, true));
    fclose($client);
}

\Co\run(function () {

    $server = stream_socket_server(
    // リッスンするプロトコルホストとポートを書き出します。
        'tcp://0.0.0.0:80',
    );

    echo "サーバーを起動しました。\n";

    // 他のコルーチンで使用できるようにチャネルを用意します。
    $channel = new \Swoole\Coroutine\Channel(
    // クライアントの最大接続数を指定します。
        100
    );

    // 接続してきたクライアントをコルーチンで処理をします
    go(function () use ($channel) {
        while (true) {
            while ($client = $channel->pop()) {
                go(fn () => parseClient($client));
            }
        }
    });

    // stream_socket_accept が false を返した場合ループが終わらないようにするためメインループとして無限ループとします。
    while (true) {
        // クライアントからの接続を待機します。
        // また、タイムアウトになると PHP Warning が発生するため stream_socket_accept は例外的にエラー制御構文を使用します。
        while ($client = @stream_socket_accept(
            // サーバーのリソースを渡します。
            $server,

            // タイムアウトは 60 秒に設定しておきます。この値は接続が来た場合は、処理が直ちに実行されるため、長めに設定することをおすすめします。
            // 規定時間内に接続がない場合 stream_socket_accept はタイムアウトして、処理が終了します。
            60
        )) {
            // 接続してきたクライアントを出力します。
            printf("%s が接続してきました。\n", stream_socket_get_name($client, true));

            $channel->push($client);
        }
    }
});
