<?php
// コルーチン内で非同期処理を有効にします。
\Swoole\Runtime::enableCoroutine();

function guaranteeRead($client, int $length) {
    // GET など、リクエストボディを送らないメソッドの場合 Content-Length が 0 になるので、早期リターンさせます。
    if ($length <= 0) {
        return '';
    }

    // 読み込み終わった文字列を格納しておくための変数を用意します。
    $read = '';

    // 残り読み込む必要のある文字列の長さを格納しておきます。
    // 本来期待する読み込む文字列の長さは $length であるため、初期値として入れておきます。
    $remaining = $length;
    do {
        // 読み込んだ値を一旦バッファという形でおいておきます。
        $buffer = fread($client, $length);

        // 読み込んだ文字列を読み込み済みのデータを変数に追記します。
        $read .= $buffer;

        // 読み込んだ文字列の長さを $remaining から減算します。
        // そうすると最終的に $remaining は 0 になるため do-while 文を抜けます。
        $remaining -= strlen($buffer);
    } while ($remaining > 0);

    return $read;
}

function parseClient($client) {
    $headers = [];
    while ($line = fgets($client)) {
        // \n または \r\n の行がでるまで繰り返します
        if ($line === "\n" || $line === "\r\n") {
            break;
        }

        $headers[] = $line;
    }

    // headers を上書きします。
    $headers = array_reduce(
        // 最初の値を含まない値を取得し、以後 key => [ value1, value2, ...valueN ] のような形にします。
        array_slice($headers, 1),
        function ($carry, $header) {

            // 読み込んだ行を配列に入れます。
            [$headerName, $headerValue] = explode(
            // : で区切るようにします。
                ':',

                // 取得した行を渡します。
                $header,

                // : の左辺と右辺のみを取得するため 2 を指定します。
                // これを指定していないと、リクエストヘッダーの値の中に : が含まれているとそこも分割されてしまうためです。
                2
            );

            // キーは左右の余分な空白を除去し、英字を小文字にします。 RFC ではリクエストヘッダー名の大小は区別しないという定義がなされているためです。[^4]
            // また値についても同様に左右の空白や改行を除去するために trim を実行します。値は Set-Cookie のように複数行渡される場合も考慮し、複数の値を保持できるようにしておきます。
            $carry[strtolower(trim($headerName))][] = trim($headerValue);
            return $carry;
        },
        // 先頭の配列は添字と値に分ける必要がないため、最初に入れておきます。
        [trim($headers[0] ?? '')]
    );

    [$method, $path, $protocol] = explode(' ', $headers[0] ?? '');

    $contentLength = max(
        // 0 以上の値にするため min を使用して 0 以上になるように保証をします。
        0,

        // Content-Length の値は文字列なためそれを int にキャストします。
        (int) ($headers['content-length'][0] ?? 0)
    );

    // 必要なリクエストが含まれていないのでクライアントとの接続を切断する
    if (!isset($headers['sec-websocket-key'], $headers['sec-websocket-version'])) {
        fclose($client);
        return;
    }

    // 未対応バージョンの場合もクライアントとの接続を切断する
    if (((int) $headers['sec-websocket-version']) === 13) {
        fclose($client);
        return;
    }

    // クライアントにレスポンスとして返すための値を生成します。
    $acceptedKey = base64_encode(
        sha1(
            // 送られてきた Sec-WebSocket-Key は base64_decode でデコードせず、そのまま使用します。
            current($headers['sec-websocket-key']) . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11',
            // PHP は SHA-1 の第二引数に true を指定することで 16 進数に変換することなくバイナリとして吐き出すことが可能です。
            true
        )
    );

    // 200 OK ではなく 101 Switching Protocols をレスポンスとして返します。
    fwrite($client, "HTTP/1.1 101 Switching Protocols\n");

    // 接続しているクライアントを WebSocket のコネクションとして扱うためのレスポンスを返します。
    fwrite($client, "Upgrade: websocket\n");
    fwrite($client, "Connection: Upgrade\n");

    // 先ほど生成した $acceptedKey を Sec-WebSocket-Accept としてレスポンスを返します。
    fwrite($client, "Sec-WebSocket-Accept: {$acceptedKey}\n");

    // HTTP の慣習として \n を最後に書き込みます。
    fwrite($client, "\n");

    // 最初の 1 バイト, FIN, RSV1~3, opcode (合計 8 ビット) を読み込みます。
    while (true) {
        while ($frame = guaranteeRead($client, 1)) {
            $frame = current(unpack('C', $frame));

            // FIN を取得します。
            $fin = $frame >> 7;

            // opcode を取得します。
            $opcode = $frame & 0b00001111;

            // MASK と Payload len を含んだ 1 バイトを取得します。
            $maskAndPayloadLength = current(unpack('C', guaranteeRead($client, 1)));

            // MASK の値を取得します。
            $mask = $maskAndPayloadLength >> 7;

            // Payload len を取得します。
            $payloadLength = $maskAndPayloadLength & 0b01111111;

            // 長さを定義します。
            $length = $payloadLength;

            // 拡張されたペイロードの長さは、加算ではなくそのまま値を上書きする点に注意してください。
            if ($payloadLength === 126) {
                // 126 の場合 unsigned short でビッグエンディアンであるため unpack の第一引数は n となります。
                $length = current(unpack('n', guaranteeRead($client, 2)));
            } elseif ($payloadLength === 127) {
                // 127 の場合 unsigned long long でビッグエンディアンであるため unpack の第一引数は J となります。
                $length = current(unpack('J', guaranteeRead($client, 8)));
            }

            $maskChars = [];
            if ($mask === 1) {
                // 4バイト取得し、それぞれの数値型を得ます。 C のあとに数値を加えた場合、指定した数値分繰り返すという定義が可能です。
                // また unpack は 1 から開始するためそれを 0 からにするため array_values を使用します。
                $maskChars = array_values(
                    unpack(
                        'C4',
                        guaranteeRead($client, 4)
                    )
                );
            }

            $payload = guaranteeRead($client, $length);

            if ($mask === 1) {
                // 指定されたペイロード分の長さから数値の配列を取得します。
                // $maskChars 同様にこちらも 0 から開始するように array_values を使用しています。
                $payload = array_values(
                    unpack(
                        "C{$length}",
                        $payload
                    )
                );

                for ($i = 0; $i < count($payload); $i++) {
                    // マスクされた値に再度マスクをすると、もともとの値を取得できます。
                    $payload[$i] = $payload[$i] ^ $maskChars[$i % 4];
                }

                // 最後に文字に治すため全ての要素に chr を実行し、最後に要素を結合します。
                $payload = implode(array_map('chr', $payload));
            }

            // 取得したペイロードを出力します。
            var_dump($payload);

            // fin + RSV1~3 と opcode を定義します。
            $responseFinAndOpCode = chr(0b10000000 + 0x01);

            // 送りたいメッセージを定義します。
            $message = "{$payload} を受け取りました！";

            // 送りたいメッセージの長さと送る種類を定義します。
            $sendType = $length = strlen($message);

            // 送る種類を文字列の長さによって分岐します。
            // 通常 65535 より大きい値は 127, 65535 以下で 126 以上の場合は 126, それ以外の場合は文字列の長さをそのまま送ることになります。
            if ($sendType > 0xffff) {
                $sendType = 127;
            } elseif ($sendType <= 0xffff && $sendType >= 126) {
                $sendType = 126;
            }

            // 2バイト目の MASK と Payload len を定義します。
            $responseMaskAndPayloadLength = chr(0b00000000 + $sendType);

            // $sendType が 127 もしくは 126 の場合、ペイロードの長さを改めて書き込みます。
            $extendedPayloadLength = '';

            // 取得する際は unpack で数値を取得していましたが、逆に数値を文字にする場合 pack を用いることで可能となります。
            if ($sendType === 127) {
                $extendedPayloadLength = pack('n', $length);
            } elseif ($sendType === 126) {
                $extendedPayloadLength = pack('J', $length);
            }

            fwrite(
                $client,
                $responseFinAndOpCode . $responseMaskAndPayloadLength . $extendedPayloadLength . $message
            );

        }
    }
}

\Co\run(function () {

    $server = stream_socket_server(
    // リッスンするプロトコルホストとポートを書き出します。
        'tcp://0.0.0.0:80',
    );

    echo "サーバーを起動しました。\n";

    // 他のコルーチンで使用できるようにチャネルを用意します。
    $channel = new \Swoole\Coroutine\Channel(
    // クライアントの最大接続数を指定します。
        100
    );

    // 接続してきたクライアントをコルーチンで処理をします
    go(function () use ($channel) {
        while (true) {
            while ($client = $channel->pop()) {
                go(fn () => parseClient($client));
            }
        }
    });

    // stream_socket_accept が false を返した場合ループが終わらないようにするためメインループとして無限ループとします。
    while (true) {
        // クライアントからの接続を待機します。
        // また、タイムアウトになると PHP Warning が発生するため stream_socket_accept は例外的にエラー制御構文を使用します。
        while ($client = @stream_socket_accept(
            // サーバーのリソースを渡します。
            $server,

            // タイムアウトは 60 秒に設定しておきます。この値は接続が来た場合は、処理が直ちに実行されるため、長めに設定することをおすすめします。
            // 規定時間内に接続がない場合 stream_socket_accept はタイムアウトして、処理が終了します。
            60
        )) {
            // 接続してきたクライアントを出力します。
            printf("%s が接続してきました。\n", stream_socket_get_name($client, true));

            $channel->push($client);
        }
    }
});
