<?php

class RoomController
{
    public function index(SwooleServerRequest $serverRequest, $params)
    {
        // データベースを操作するための PDO を取得します。
        $pdo = Connection::factory()
            ->getInstance();

        // ルームの一覧を取得します。
        $statement = $pdo->prepare(
            'SELECT * FROM `rooms`'
        );

        // プリペアドステートメントを実行します。
        $statement->execute();

        // 全ルームを取得します。
        $results = $statement->fetchAll(PDO::FETCH_OBJ);

        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/TemplateRooms.php', [
                'rooms' => $results,
                'user' => $serverRequest->getSession()->read('user'),
                'error_code' => (int)($serverRequest->getQueryParams()['error_code'] ?? 0),
            ])
        );
    }

    public function add(SwooleServerRequest $serverRequest, $params)
    {
        // POST で送られてきたときの処理
        if ($serverRequest->getMethod() === 'POST') {
            // POST されたデータを変数に格納します。
            $post = $serverRequest->getParsedBody();

            // ルーム名は1文字以上32文字以下とするため、それ以外の値の場合、エラーとするようにします。
            $name = $post['name'];
            if (mb_strlen($name) < 1 || mb_strlen($name) > 32) {
                // error を 1 としてリダイレクトさせます。
                return (new SwooleResponse($serverRequest, ''))
                    // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                    ->withStatus(301)
                    ->withHeader(
                        'location',
                        "/rooms?error_code=1"
                    );
            }

            // エラーがなければ保存処理を行います。
            // データベースを操作するための PDO を取得します。
            $pdo = Connection::factory()
                ->getInstance();

            // ルームを作成します。
            $statement = $pdo->prepare(
                'INSERT INTO `rooms` (name, user_id, created_at, updated_at) VALUES (?, ?, ?, ?)'
            );

            $user = $serverRequest->getSession()->read('user');
            $statement->bindValue(1, $name);
            $statement->bindValue(2, $user->id);
            $statement->bindValue(3, date('Y-m-d H:i:s'));
            $statement->bindValue(4, date('Y-m-d H:i:s'));

            // プリペアドステートメントを実行します。
            $statement->execute();
        }

        return (new SwooleResponse($serverRequest, ''))
            // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
            ->withStatus(301)
            ->withHeader(
                'location',
                "/rooms"
            );
    }

    public function details(SwooleServerRequest $serverRequest, $params)
    {
        // 指定された ID を取得します。
        $id = $params['id'] ?? null;

        if ($id === null) {
            return (new SwooleResponse($serverRequest, ''))
                // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                ->withStatus(301)
                ->withHeader(
                    'location',

                    // エラーコードは 10 としておきます。
                    "/rooms?error_code=10"
                );
        }

        // データベースを操作するための PDO を取得します。
        $pdo = Connection::factory()
            ->getInstance();

        // 指定された ID でルームを取得します。
        $statement = $pdo->prepare(
            'SELECT * FROM `rooms` WHERE id=?'
        );

        $statement->bindValue(1, (int) $id, PDO::PARAM_INT);

        // プリペアドステートメントを実行します。
        $statement->execute();

        // 全ルームを取得します。
        $room = $statement->fetch(PDO::FETCH_OBJ);

        // カーソルを閉じます。
        $statement->closeCursor();

        // ルームがなければエラーを返します。
        if (!$room) {
            return (new SwooleResponse($serverRequest, ''))
                // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                ->withStatus(301)
                ->withHeader(
                    'location',

                    // エラーコードは 10 としておきます。
                    "/rooms?error_code=10"
                );
        }

        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/TemplateRoomId.php', [
                'room' => $room,
                'user' => $serverRequest->getSession()->read('user'),
            ])
        );
    }

    public function delete(SwooleServerRequest $serverRequest, $params)
    {
        // 指定された ID を取得します。
        $id = $params['id'] ?? null;

        if ($id === null) {
            return (new SwooleResponse($serverRequest, ''))
                // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                ->withStatus(301)
                ->withHeader(
                    'location',

                    // エラーコードは 10 としておきます。
                    "/rooms?error_code=10"
                );
        }

        // データベースを操作するための PDO を取得します。
        $pdo = Connection::factory()
            ->getInstance();


        // 指定された ID でルームを取得します。
        $statement = $pdo->prepare(
            'DELETE FROM `rooms` WHERE id=? AND user_id=?'
        );

        $statement->bindValue(1, (int) $id, PDO::PARAM_INT);

        // 自分がユーザーであるもの以外は削除しないようにします。
        $statement->bindValue(2, $serverRequest->getSession()->read('user')->id, PDO::PARAM_INT);

        // プリペアドステートメントを実行します。
        $statement->execute();

        return (new SwooleResponse($serverRequest, ''))
            // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
            ->withStatus(301)
            ->withHeader(
                'location',
                "/rooms"
            );
    }
}
