<?php


class SwooleResponse implements \Psr\Http\Message\ResponseInterface
{
    protected \Psr\Http\Message\ServerRequestInterface $serverRequest;
    protected \Psr\Http\Message\StreamInterface $response;

    protected $protocol = 'HTTP/1.1';
    protected array $headers = [];
    protected array $status = [200, 'OK'];

    public function __construct(\Psr\Http\Message\ServerRequestInterface $serverRequest, string $response)
    {
        $this->serverRequest = $serverRequest;
        $this->response = new SwooleBodyStream($response);
    }

    /**
     * レスポンスヘッダに指定している HTTP のバージョンを返すためのメソッドです。例えば HTTP/1.1 などです。
     */
    public function getProtocolVersion()
    {
        return $this->protocol;
    }

    /**
     * レスポンスヘッダの HTTP のバージョンを指定するためのメソッドです。
     */
    public function withProtocolVersion($version)
    {
        $this->protocol = $version;
        return $this;
    }

    /**
     * レスポンスヘッダーに設定されている値を取得するためのメソッドです。
     */
    public function getHeaders()
    {
        return $this->headers;
    }

    /**
     * レスポンスヘッダに第一引数で指定した値が含まれているかどうかを返すためのメソッドです。
     */
    public function hasHeader($name)
    {
        return isset($this->headers[$name]);
    }

    /**
     * レスポンスヘッダに第一引数で指定した値を返すためのメソッドです。
     */
    public function getHeader($name)
    {
        return $this->headers[$name];
    }

    /**
     * 指定した名前のレスポンスヘッダを返すためのメソッドです。
     * getHeader メソッドとの違いは，同名のレスポンスヘッダが複数あっても1つの文字列として返す点です。
     */
    public function getHeaderLine($name)
    {
        $header = $this->getHeader($name);
        return is_array($header)
            ? implode(', ', $header)
            : $header;
    }

    /**
     * 新しくレスポンスヘッダを加える（追加する）ためのメソッドです。
     */
    public function withHeader($name, $value)
    {
        $this->headers[$name] = $value;
        return $this;
    }

    /**
     * 新しくレスポンスヘッダを加える（追加する）ためのメソッドです。
     * withHeader との相違点は，既に同名のリクエストが存在する場合は，上書きせず，配列の要素として追加をする点です。
     */
    public function withAddedHeader($name, $value)
    {
        $name = strtolower($name);
        if ($this->hasHeader($name)) {
            $this->headers[$name][] = $value;
            return $this;
        }
        return $this->withHeader($name, $value);
    }

    /**
     * 指定したレスポンスヘッダーを削除するためのメソッドです。
     */
    public function withoutHeader($name)
    {
        unset($this->headers[$name]);
        return $this;
    }

    /**
     * レスポンスボディを取得するためのメソッドです。
     */
    public function getBody()
    {
        return $this->response;
    }

    /**
     * レスポンスボディを別のものに置き換えるためのメソッドです。
     */
    public function withBody(\Psr\Http\Message\StreamInterface $body)
    {
        $this->response = $body;
        return $this;
    }

    /**
     * レスポンスとして返すためのステータスコードを取得するメソッドです。
     */
    public function getStatusCode()
    {
        return $this->status[0];
    }

    /**
     * レスポンスとして返すためのステータスコードを変更するためのメソッドです。
     */
    public function withStatus($code, $reasonPhrase = '')
    {
        $this->status = [$code, $reasonPhrase];
        return $this;
    }

    /**
     * レスポンスとして返すためのステータスコードの理由を取得するためのメソッドです。
     */
    public function getReasonPhrase()
    {
        return $this->status[1];
    }
}
