<?php
require __DIR__ . '/vendor/autoload.php';

class SwooleServerRequest implements \Psr\Http\Message\ServerRequestInterface
{
    protected $body;
    protected $requestTarget;
    protected $uri;

    protected \Swoole\Server $swooleServer;
    protected \Swoole\Http\Request $swooleHttpRequest;
    protected \Swoole\Http\Response $swooleHttpResponse;

    public function __construct(\Swoole\Server $swooleServer, \Swoole\Http\Request $swooleHttpRequest, \Swoole\Http\Response $swooleHttpResponse)
    {
        // 以下はプロパティに値を代入しています
        $this->swooleServer = $swooleServer;
        $this->swooleHttpRequest = $swooleHttpRequest;
        $this->swooleHttpResponse = $swooleHttpResponse;

        // SwooleUri クラスをインスタンス化し，リクエストされた URI (リソースの場所)を第一引数として渡します。
        $this->uri = new SwooleUri($this->swooleHttpRequest->server['request_uri'] ?? '/');
    }

    /**
     * HTTP のバージョンを返すためのメソッドです。例えば HTTP/1.1 などです。
     */
    public function getProtocolVersion()
    {
        return $this->swooleHttpRequest->server['server_protocol'];
    }

    /**
     * HTTP のバージョンを任意の値（第一引数から渡された値）に変更するためメソッドです。
     */
    public function withProtocolVersion($version)
    {
        $this->swooleHttpRequest->server['server_protocol'] = $version;
    }

    /**
     * リクエストヘッダの一蘭を取得するためのメソッドです。
     */
    public function getHeaders()
    {
        return $this->swooleHttpRequest->header;
    }

    /**
     * 指定した名前のリクエストヘッダが含まれているかを確認するためのメソッドです。
     */
    public function hasHeader($name)
    {
        return isset($this->swooleHttpRequest->header[strtolower($name)]);
    }

    /**
     * 指定した名前のリクエストヘッダを返すためのメソッドです。同名のリクエストヘッダがある場合は，配列として返します。
     */
    public function getHeader($name)
    {
        return $this->swooleHttpRequest
            ->header[strtolower($name)];
    }

    /**
     * 指定した名前のリクエストヘッダを返すためのメソッドです。
     * getHeader メソッドとの違いは，同名のリクエストヘッダが複数あっても1つの文字列として返す点です。
     */
    public function getHeaderLine($name)
    {
        $header = $this->getHeader($name);
        return is_array($header)
            ? implode(', ', $header)
            : $header;
    }

    /**
     * 新しくリクエストヘッダを加える（追加する）ためのメソッドです。
     */
    public function withHeader($name, $value)
    {
        $this->swooleHttpRequest
            ->header[strtolower($name)] = $value;
        return $this;
    }

    /**
     * 新しくリクエストヘッダを加える（追加する）ためのメソッドです。
     * withHeader との相違点は，既に同名のリクエストが存在する場合は，上書きせず，配列の要素として追加をする点です。
     */
    public function withAddedHeader($name, $value)
    {
        $name = strtolower($name);
        if ($this->hasHeader($name)) {
            $this->swooleHttpRequest
                ->header[$name][] = $value;
            return $this;
        }
        return $this->withHeader($name, $value);
    }

    /**
     * 指定したリクエストヘッダーを削除するためのメソッドです。
     */
    public function withoutHeader($name)
    {
        unset($this->swooleHttpRequest
                ->header[$name]);
        return $this;
    }

    /**
     * リクエストボディを取得するためのメソッドです。
     */
    public function getBody()
    {
        return $this->body ?? new SwooleBodyStream(
                $this->swooleHttpRequest->getData()
            );
    }

    /**
     * リクエストボディを別のものに置き換えるためのメソッドです。
     */
    public function withBody(\Psr\Http\Message\StreamInterface $body)
    {
        $this->body = $body;
        return $this;
    }

    /**
     * リクエストヘッダからリクエストしたいリソース（パス）を取得するためのメソッドです
     */
    public function getRequestTarget()
    {
        return $this->requestTarget ?? parse_url(
                $this->swooleHttpRequest->server['request_uri'] ?? '/',
                PHP_URL_PATH
            );
    }

    /**
     * リクエストしたいリソース（パス）を指定したリソース（パス）に変更するためのメソッドです。
     */
    public function withRequestTarget($requestTarget)
    {
        $this->requestTarget = $requestTarget;
        return $this;
    }

    /**
     * GET や POST などのリクエストメソッドを取得するためのメソッドです。
     */
    public function getMethod()
    {
        return $this->swooleHttpRequest->server['request_method'];
    }

    /**
     * リクエストメソッドを第一引数で指定した任意の値に変更するためのメソッドです。
     */
    public function withMethod($method)
    {
        $this->swooleHttpRequest->server['request_method'] = $method;
        return $this;
    }

    /**
     * コンストラクタでインスタンス化した SwooleUri の値を返すためのメソッドです。
     */
    public function getUri()
    {
        return $this->uri;
    }

    /**
     * コンストラクタでインスタンス化した SwooleUri の値を別のものに置き換えるためのメソッドです。
     */
    public function withUri(\Psr\Http\Message\UriInterface $uri, $preserveHost = false)
    {
        $this->uri = new SwooleUri($uri);
        return $this;
    }

    /**
     * PHP の $_SERVER (https://www.php.net/manual/ja/reserved.variables.server.php) と同じ値を返すためのメソッドです。
     */
    public function getServerParams()
    {
        // $_SERVER と同じ内容を $swooleServer と $swooleHttpRequest から生成します。
        return [
            'PHP_SELF' => $this->getRequestTarget(),
            'SERVER_ADDR' => $this->swooleServer->host,
            'SERVER_NAME' => $this->swooleServer->host,
            'SERVER_SOFTWARE' => 'swoole',
            'SERVER_PROTOCOL' => $this->getProtocolVersion(),
            'REQUEST_METHOD' => $this->getMethod(),
            'REQUEST_TIME' => $this->swooleHttpRequest->server->master_time,
            'REQUEST_TIME_FLOAT' => $this->swooleHttpRequest->server->request_time_float,
            'QUERY_STRING' => $this->swooleHttpRequest->server->query_string ?? '',
            'DOCUMENT_ROOT' => $this->swooleHttpRequest->server->path_info,
            'HTTP_ACCEPT' => $this->getHeader('accept'),
            'HTTP_ACCEPT_ENCODING' => $this->getHeader('accept-encoding'),
            'HTTP_ACCEPT_LANGUAGE' => $this->getHeader('accept-language'),
            'HTTP_CONNECTION' => $this->getHeader('connection') ?? 'closed',
            'HTTP_HOST' => $this->getHeader('host') ?? 'localhost',
            'HTTP_REFERER' => $this->getHeader('referer') ?? '',
            'HTTP_USER_AGENT' => $this->getHeader('user-agent') ?? '',
            'HTTPS' => '',
            'REMOTE_ADDR' => $this->swooleHttpRequest->server->remote_addr,
            'REMOTE_PORT' => $this->swooleHttpRequest->server->remote_port,
            'SERVER_PORT' => $this->swooleServer->port,
            'SCRIPT_NAME' => __FILE__,
            'REQUEST_URI' => (string) $this->getUri(),
        ];
    }

    /**
     * クッキーの情報を取得するためのメソッドです。
     */
    public function getCookieParams()
    {
        return $this->swooleHttpRequest->cookie ?? [];
    }

    /**
     * クッキーの値を上書きするためのメソッドです。
     */
    public function withCookieParams(array $cookies)
    {
        $this->swooleHttpRequest->cookie = $cookies;
        return $this;
    }

    /**
     * リクエストされたクエリ文字列を配列で取得するためのメソッドです。
     */
    public function getQueryParams()
    {
        return $this->swooleHttpRequest->get ?? [];
    }

    /**
     * 指定したクエリ文字列を第一引数に渡した任意の配列に置き換えるためのメソッドです。
     */
    public function withQueryParams(array $query)
    {
        $this->swooleHttpRequest->get = $query;
        return $this;
    }

    /**
     * クライアントからアップロードされたファイルの一蘭を取得するためのメソッドです。
     */
    public function getUploadedFiles()
    {
        return $this->swooleHttpRequest->files ?? [];
    }

    /**
     * アップロードされたファイルを第一引数で指定した任意の配列に置き換えるためのメソッドです。
     */
    public function withUploadedFiles(array $uploadedFiles)
    {
        $this->swooleHttpRequest->files = $uploadedFiles;
        return $this;
    }

    /**
     * クライアントから送られてきた POST の値を取得するためのメソッドです。
     */
    public function getParsedBody()
    {
        return $this->swooleHttpRequest->post ?? [];
    }

    /**
     * POST の値を第一引数で指定した任意の値に置き換えるためのメソッドです。
     */
    public function withParsedBody($data)
    {
        $this->swooleHttpRequest->post = $data;
        return $this;
    }

    /**
     * 今回は特に使用しないので null を返すようにします。
     */
    public function getAttributes()
    {
        return null;
    }

    /**
     * 今回は特に使用しないので null を返すようにします。
     */
    public function getAttribute($name, $default = null)
    {
        return null;
    }

    /**
     * 今回は特に使用しないのでそのまま現状のオブジェクトを返します。
     */
    public function withAttribute($name, $value)
    {
        return $this;
    }

    /**
     * 今回は特に使用しないのでそのまま現状のオブジェクトを返します。
     */
    public function withoutAttribute($name)
    {
        return $this;
    }

    /**
     * SwooleResponse オブジェクトを返すためのメソッドです。
     */
    public function getResponse()
    {
        return $this->swooleHttpResponse;
    }

    /**
     * セッションのインスタンス化したオブジェクトを返すためのメソッドです。
     */
    public function getSession(): Session
    {
        return new Session($this);
    }
}
