<?php
require __DIR__ . '/vendor/autoload.php';

Swoole\Runtime::enableCoroutine();

function loadTemplate(string $path, array $templateVars = []) {
    // 予め読み込んだテンプレートファイルを保持しておくための変数を定義しておきます。
    static $loadedTemplates = [];

    // 現在のディレクトリからのパスとして扱います。
    $path = __DIR__ . '/' . $path;

    // $templateVars で定義されているものを変数として扱えるようにします。
    extract($templateVars);

    // 既に定義されている場合、ファイルを読み込まず、そのまま返すようにします。
    if (isset($loadedTemplates[$path])) {
        // PHP として実行するために ob_start/ob_get_clean と eval を使用します。
        ob_start();
        eval(' ?>' . $loadedTemplates[$path] . '<?php ');
        return ob_get_clean();
    }

    // ファイルが存在する場合、テンプレートファイルの中身を返します。
    if (is_file($path)) {
        $loadedTemplates[$path] = file_get_contents($path);

        // PHP として実行するために ob_start/ob_get_clean と eval を使用します。
        ob_start();
        eval(' ?>' . $loadedTemplates[$path] . '<?php ');
        return ob_get_clean();
    }

    // テンプレートファイルが存在しない場合、エラーを出力します。
    return "`{$path}` というテンプレートファイルは存在しません。";
}

$server = new \Swoole\WebSocket\Server(
    '0.0.0.0',
    80,
);

$server->set([
    'open_http_protocol' => true
]);

$server->on(
    'start',
    function () {
        echo "Server started!\n";
    }
);

$server->on(
    'request',
    function (\Swoole\Http\Request $request, \Swoole\Http\Response $response) use ($server) {

        // \League\Route\Router をインスタンス化します
        $router = new \League\Route\Router();

        // /favicon.ico で接続してきた場合、空のレスポンスを返すようにします。
        $router->get('/favicon.ico', function (\Psr\Http\Message\ServerRequestInterface $serverRequest) {
            return new SwooleResponse(
                $serverRequest,
                ''
            );
        });

        $swooleServerRequest = new SwooleServerRequest($server, $request, $response);

        // / に接続してきた場合「Hello World!」を出力するようにします。
        $router->get('/', 'MainController::index');
        $router->get('/register', 'RegisterController::index');
        $router->post('/register', 'RegisterController::post');
        $router->get('/register/success', 'RegisterController::success');
        $router->get('/login', 'LoginController::index');
        $router->post('/login', 'LoginController::login');

        $router->get('/profile/{id}', 'ProfileController::index');

        // ログインしている場合
        if (!!$swooleServerRequest->getSession()->read('user')) {
            // ルーム一覧用ページ
            $router->post('/rooms/delete/{id}', 'RoomController::delete');
            $router->get('/rooms/{id}', 'RoomController::details');
            $router->get('/rooms', 'RoomController::index');
            $router->post('/rooms/add', 'RoomController::add');
        }

        try {
            // \League\Route\Router::dispatch でリクエストを処理します。
            $routerDispatched = $router->dispatch($swooleServerRequest);
        } catch (\League\Route\Http\Exception\BadRequestException $e) {
            // BadRequestException が投げられた際に不正なリクエストとして処理をします。
            $response->setStatusCode(400);
            $response->end(loadTemplate('templates/Error.php', [
                'message' => "不正なリクエストです。",
            ]));
            return;
        } catch (\League\Route\Http\Exception\UnauthorizedException $e) {
            // UnauthorizedException が投げられた際に認証を失敗した旨を表示します。
            $response->setStatusCode(401);
            $response->end(loadTemplate('templates/Error.php', [
                'message' => "認証に失敗しました。",
            ]));
            return;
        } catch (\League\Route\Http\Exception\ForbiddenException $e) {
            // Forbidden
            $response->setStatusCode(403);
            $response->end(loadTemplate('templates/Error.php', [
                'message' => "ページへアクセスする権限がありません。",
            ]));
            return;
        } catch (\League\Route\Http\Exception\NotFoundException $e) {
            // NotFoundException が投げられた際にページが存在しない旨を表示します。
            $response->setStatusCode(404);
            $response->end(loadTemplate('templates/Error.php', [
                'message' => "ページが存在しません。",
            ]));
            return;
        } catch (Throwable $e) {
            // 拾いきれなかったエラーなどをまとめて処理します
            $response->setStatusCode(500);
            var_dump($e);
            $response->end(loadTemplate('templates/Error.php', [
                'message' => "サーバー内部でエラーが発生しています。",
            ]));
            return;
        }

        // HTTP ステータスコードをセットします。
        $response->setStatusCode(
            $routerDispatched->getStatusCode()
        );

        // レスポンスヘッダーをセットします。
        foreach ($routerDispatched->getHeaders() as $key => $value) {
            $response->header($key, $value, true);
        }

        // レスポンスを返すようにします。
        $response->end(
            $routerDispatched
                ->getBody()
                ->getContents()
        );

    }
);

// 変数は別のコルーチンから影響を受けないため、下記のようにあえて変数を使用します。
$clients = [];
$server->on('open', function (\Swoole\Http\Server $server, \Swoole\Http\Request $request) use (&$clients) {
    echo "Connection opened: {$request->fd}\n";
    $client = new SwooleServerRequest($server, $request, new \Swoole\Http\Response());

    // ユーザーがログインしているかどうかを確認します。
    if (!!$client->getSession()->read('user')) {
        // ログイン中であれば $clients に追加します。
        $clients[$request->fd] = $client;
    }
});

// 受け取ったメッセージを接続中の全クライアントにブロードキャストします。
$server->on('message', function (\Swoole\WebSOcket\Server $server, \Swoole\WebSocket\Frame $frame) use (&$clients) {
    foreach ($server->connections as $fd) {
        // WebSocket 用の接続ではない場合は、送らないようにします。
        if ($server->connection_info($fd)['websocket_status'] === 0) {
            continue;
        }
        $server->push(
            $fd,
            json_encode([
                'name' => $clients[$frame->fd]->getSession()->read('user')->name,

                // HTML タグなどが送られても問題ないようにしています。
                'message' => htmlspecialchars($frame->data, ENT_QUOTES, 'UTF-8'),
                'date' => date('Y-m-d H:i:s'),
            ]),
        );
    }
});

// 切断されたクライアントは切断処理をします。
$server->on('close', function (\Swoole\Http\Server $server, int $fd) use (&$clients) {
    unset($clients[$fd]);
});

$server->start();
